using Newtonsoft.Json;
using System;
using System.Collections;
using System.Numerics;
using System.Collections.Generic;
using System.Data;
using System.Drawing;
using System.IO;
using System.Reflection;
using System.Security.Cryptography.X509Certificates;
using GrapeCity.Documents.Excel;
using GrapeCity.Documents.Excel.Forms;
using GrapeCity.Documents.Excel.Expressions;
using GrapeCity.Documents.Excel.Drawing;
using System.Globalization;
using System.Text.RegularExpressions;

namespace GrapeCity.Documents.Excel.Examples.Features.DataBinding.NumberFormat
{
    class Program
    {
        static void Main(string[] args)
        {
            //create a new workbook
            var workbook = new GrapeCity.Documents.Excel.Workbook();

            // Get data from json file.

            string jsonText = string.Empty;
            using (Stream stream = GetResourceStream("json\\DataBinding.json"))
            using (StreamReader reader = new StreamReader(stream))
            {
                jsonText = reader.ReadToEnd();
            }

            IWorksheet worksheet = workbook.ActiveSheet;

            //Import Dates stored as texts to a Date format

            worksheet.DataSource = new JsonDataSource(jsonText);            
            worksheet.Columns[5].NumberFormat = "dd/mm/yyyy";
            worksheet.Columns[6].NumberFormat = "d/m/yy";
            worksheet.Columns.AutoFit();

            //Show big numbers in a format other than scientific format
            workbook.Worksheets.Add().Name = "Big Number";
            workbook.Worksheets["Big Number"].Range["A1"].Value = "From Data";
            workbook.Worksheets["Big Number"].Range["B1"].Value = "Converted to number";            

            workbook.Worksheets["Big Number"].Range["A2:B2"].Value = 6589423145697;            
            workbook.Worksheets["Big Number"].Range["B2"].NumberFormat = "0";
            workbook.Worksheets["Big Number"].Columns.AutoFit();


            //Show leading or trailing zeros
            workbook.Worksheets.Add().Name = "Zeros";
            workbook.Worksheets["Zeros"].Range["A1"].Value = "Actual value:";
            workbook.Worksheets["Zeros"].Range["B1"].Value = "`00123";
            workbook.Worksheets["Zeros"].Range["A2"].Value = "In Excel";
            workbook.Worksheets["Zeros"].Range["B2"].Value = 00123;
            workbook.Worksheets["Zeros"].Range["A3"].Value = "After number format";
            workbook.Worksheets["Zeros"].Range["B3"].Value = 00123;
            workbook.Worksheets["Zeros"].Range["B3"].NumberFormat = "00000";

            workbook.Worksheets["Zeros"].Range["A6"].Value = "Actual value:";
            workbook.Worksheets["Zeros"].Range["B6"].Value = "`1234.5600";
            workbook.Worksheets["Zeros"].Range["A7"].Value = "In Excel";
            workbook.Worksheets["Zeros"].Range["B7"].Value = 1234.5600;
            workbook.Worksheets["Zeros"].Range["A8"].Value = "After number format";
            workbook.Worksheets["Zeros"].Range["B8"].Value = 1234.5600;
            workbook.Worksheets["Zeros"].Range["B8"].NumberFormat = "#.0000";
            workbook.Worksheets["Zeros"].Columns.AutoFit();

            //Conditional Formatting

            object[,] data0 = new object[,]{
                {"Name", "City", "Result"},
                {"Richard", "New York", 10 },
                {"Nia", "New York", 5},
                {"Jared", "New York",-6 },
                {"Natalie", "Washington",-2 },
                {"Damon", "Washington", "Unimpressive" },
                {"Angela", "Washington", "Unimpressive" }
            };

            workbook.Worksheets.Add().Name = "Conditional Format";
            workbook.Worksheets["Conditional Format"].Range["A1:C7"].Value = data0;            
            workbook.Worksheets["Conditional Format"].Range["C2:C7"].NumberFormat = "General;[Red]-General;;[Blue]General";
            workbook.Worksheets["Conditional Format"].Columns.AutoFit();


            //Disguise Numbers as Text
            object[,] data1 = new object[,]{
                {"Name", "Subject", "Result"},
                {"Richard", "English", 70 },
                {"Richard", "Elective-I", 65},
                {"Richard", "Science",45 },
                {"Richard", "Mathematics",56 },
                {"Richard", "Elective-II", 25 },
                {"Richard", "Language", 38 },
                {"Natalie", "English", 30 },
                {"Natalie", "Elective-I", 45},
                {"Natalie", "Science",53 },
                {"Natalie", "Mathematics",28 },
                {"Natalie", "Elective-II", 32 },
                {"Natalie", "Language", 69 }
            };

            workbook.Worksheets.Add().Name = "DisguiseNumber";
            workbook.Worksheets["DisguiseNumber"].Range["A1:C13"].Value = data1;
            workbook.Worksheets["DisguiseNumber"].Range["C2:C13"].NumberFormat = "[<35][Red]\"Fail\";[Green]\"Pass\"";
            workbook.Worksheets["DisguiseNumber"].Columns.AutoFit();


            //Format number as Thousand/Million/Billion

            workbook.Worksheets.Add().Name = "NumberScale";

            workbook.Worksheets["NumberScale"].Range["A1"].Value = "Number scale Format";
            workbook.Worksheets["NumberScale"].Range["A1:D1"].MergeCells = true;

            workbook.Worksheets["NumberScale"].Range["A3"].Value = "Number";
            workbook.Worksheets["NumberScale"].Range["B3"].Value = "Thousands";
            workbook.Worksheets["NumberScale"].Range["C3"].Value = "Million";
            workbook.Worksheets["NumberScale"].Range["D3"].Value = "Billion";

            workbook.Worksheets["NumberScale"].Range["A4:D4"].Value = 456321456;
            workbook.Worksheets["NumberScale"].Range["B4"].NumberFormat = "0.00, \"K\"";    
            workbook.Worksheets["NumberScale"].Range["C4"].NumberFormat = "$0.00,, \"M\"";    
            workbook.Worksheets["NumberScale"].Range["D4"].NumberFormat = "$#.##,,, \"B\"";
            workbook.Worksheets["NumberScale"].Columns.AutoFit();

            //Culture Specific data
            workbook.Worksheets.Add().Name = "Culture";
              
            
            workbook.Worksheets["Culture"].Range["A1"].Value = "Date as Text";
            workbook.Worksheets["Culture"].Range["A2"].Value = "12/27/2022";
            workbook.Worksheets["Culture"].Range["A3"].Value = "08/07/2022";

            workbook.Culture = CultureInfo.GetCultureInfo("en-US"); //set culture of data available
            workbook.AutoParse = true;

            workbook.Worksheets["Culture"].Range["B1"].Value = "Date converted to system locale (not US)";
            workbook.Worksheets["Culture"].Range["B2"].Value = "12/27/2022";
            workbook.Worksheets["Culture"].Range["B3"].Value = "08/07/2022";

            workbook.Worksheets["Culture"].Range["C1"].Value = "Formatted to US date format";
            workbook.Worksheets["Culture"].Range["C2"].Value = "12/27/2022";
            workbook.Worksheets["Culture"].Range["C3"].Value = "08/07/2022";
            workbook.Worksheets["Culture"].Range["C2:C3"].NumberFormat = "[$-en-US]mm-dd-yyyy";

            workbook.Worksheets["Culture"].Columns.AutoFit();


            //Display decimals as Fractions
            workbook.Worksheets.Add().Name = "Fractions";

            workbook.Worksheets["Fractions"].Range["A1"].Value = "Decimal";
            workbook.Worksheets["Fractions"].Range["B1"].Value = "# ##/##";
            workbook.Worksheets["Fractions"].Range["C1"].Value = "###/###";
            workbook.Worksheets["Fractions"].Range["D1"].Value = "##/4";


            workbook.Worksheets["Fractions"].Range["A2:D2"].Value = 0.25;         
            workbook.Worksheets["Fractions"].Range["A3:D3"].Value = 5.64;     
            workbook.Worksheets["Fractions"].Range["A4:D4"].Value = 2.5;

            workbook.Worksheets["Fractions"].Range["B2:B4"].NumberFormat = "# ##/##";
            workbook.Worksheets["Fractions"].Range["C2:C4"].NumberFormat = "###/###";
            workbook.Worksheets["Fractions"].Range["D2:D4"].NumberFormat = "##/4";

            workbook.Worksheets["Fractions"].Columns.AutoFit();


            workbook.Save("ConvertedNumberFormat.xlsx");
        }

        static Stream GetResourceStream(string resourcePath)
        {
            string resource = "NumberFormat.Resource." + resourcePath.Replace("\\", ".");
            var assembly = typeof(Program).GetTypeInfo().Assembly;
            return assembly.GetManifestResourceStream(resource);
        }

    }
}